import os
import json
import requests
import urllib3
from deepsecurity.rest import ApiException


def value_exists(some_json, key):
    value = some_json.get(key)
    if value is not None:
        return value
    else:
        return False


def rename_json(json):
    if json["name"]:
        json["name"] = f"{json['name']} - Migrated"
    return json


def to_json_name(snake_case):
    temp = str(snake_case).split("_")
    json_name = temp[0] + "".join(ele.title() for ele in temp[1:])
    return json_name


def to_api_endpoint(snake_case):
    temp = str(snake_case).split("_")
    endpoint_name = "".join(ele for ele in temp)
    return endpoint_name


def to_title(snake_case):
    temp = str(snake_case).split("_")
    title = " ".join(ele.title() for ele in temp)
    return title


def http_list_get(type, OLD_HOST, OLD_API_KEY, cert=False):
    """
    Takes in a string type in snake case, outputs a list of string json (for now)
    """
    list_all = []
    list_name = []
    list_id = []
    endpoint = f"{OLD_HOST}api/{to_api_endpoint(type)}"
    headers = {
        "api-secret-key": OLD_API_KEY,
        "api-version": "v1",
        "Content-Type": "application/json",
    }
    response = requests.request("GET", endpoint, headers=headers, data={}, verify=cert)
    describe = str(response.text)
    response_json = json.loads(describe).get(to_json_name(type))
    if response_json is not None:
        for count, item in enumerate(response_json):
            list_all.append(str(json.dumps(item)))
            list_id.append(str(item["ID"]))
            print(
                f"#{str(count)} {to_title(type)} name: {str(item['name'])}", flush=True
            )
            print(f"#{str(count)} {to_title(type)} ID: {str(item['ID'])}", flush=True)
        print("Done!", flush=True)
    return list_all, list_name, list_id


def http_search(
    type,
    list_name,
    OLD_HOST,
    OLD_API_KEY,
    cert=False,
    search_type="name",
):
    # returns a list of items
    all_results = []
    if not isinstance(list_name, list):
        list_name = [list_name]
    for item in list_name:
        payload = (
            '{"searchCriteria": [{"fieldName": "'
            + str(search_type)
            + '","stringValue": "'
            + str(item)
            + '"}]}'
        )
        endpoint = f"{OLD_HOST}api/{to_api_endpoint(type)}/search"
        headers = {
            "api-secret-key": OLD_API_KEY,
            "api-version": "v1",
            "Content-Type": "application/json",
        }
        response = requests.request(
            "POST", endpoint, headers=headers, data=payload, verify=cert
        )
        describe = str(response.text)
        all_results.append(describe)
    return all_results


def validate_create(all_old, api_instance, type):
    all_new = []
    for count, dirlist in enumerate(all_old):
        namecheck = 1
        rename = 1
        oldjson = json.loads(dirlist)
        oldname = oldjson["name"]
        while namecheck != -1:
            try:
                newname = api_instance.create(oldjson)
                newid = api_instance.search(newname)
                print(
                    "#"
                    + str(count)
                    + " "
                    + type.capitalize()
                    + " List ID: "
                    + str(newid)
                    + ", Name: "
                    + newname,
                    flush=True,
                )
                all_new.append(str(newid))
                namecheck = -1
            except ApiException as e:
                error_json = json.loads(e.body)
                if (
                    "requested scheduled task name already exists"
                    in error_json["message"]
                ):
                    oldjson["name"] = oldname + " {" + str(rename) + "}"
                    rename = rename + 1
                elif (
                    "name already exists"
                    or "Name must be unique" in error_json["message"]
                ):
                    print(
                        f"{oldjson['name']} already exists in new tenant, renaming...",
                        flush=True,
                    )
                    oldjson["name"] = oldname + " {" + str(rename) + "}"
                    rename = rename + 1
                else:
                    print(e.body, flush=True)
                    namecheck = -1
    return all_new


def validate_create_dict(all_old, api_instance, type):
    new_dict = {}
    for count, dirlist in enumerate(all_old):
        namecheck = 1
        rename = 1
        oldjson = json.loads(dirlist)
        oldname = oldjson["name"]
        oldid = oldjson["ID"]
        while namecheck != -1:
            try:
                newname = api_instance.create(oldjson)
                newid = api_instance.search(newname)
                print(
                    "#"
                    + str(count)
                    + " "
                    + type.capitalize()
                    + " List ID: "
                    + str(newid)
                    + ", Name: "
                    + newname,
                    flush=True,
                )
                new_dict[oldid] = newid
                namecheck = -1
            except ApiException as e:
                error_json = json.loads(e.body)
                if "name already exists" in error_json["message"]:
                    print(
                        f"{oldjson['name']} already exists in new tenant, renaming..."
                    )
                    oldjson["name"] = oldname + " {" + str(rename) + "}"
                    rename = rename + 1
                else:
                    print(e.body, flush=True)
                    namecheck = -1
    return new_dict



import sys
import os
import requests
import urllib3
import json
from dsmigrator.api_config import SystemSettingsApiInstance
from deepsecurity.rest import ApiException

cert = False


def ListSettings(url_link_final, tenant1key):
    payload = {}
    url = url_link_final + "api/systemsettings"
    headers = {
        "api-secret-key": tenant1key,
        "api-version": "v1",
        "Content-Type": "application/json",
    }
    response = requests.request("GET", url, headers=headers, data=payload, verify=cert)
    describe = str(response.text)
    settings_dict = json.loads(describe)
    # print(settings_dict)
    return settings_dict


no_fly_list = [
    "platformSettingAgentInitiatedActivationToken",
    "platformSettingApiSoapWebServiceEnabled",
    "platformSettingApiStatusMonitoringEnabled",
    "platformSettingAzureSsoCertificate",
    "platformSettingContentSecurityPolicy",
    "platformSettingContentSecurityPolicyReportOnlyEnabled",
    "platformSettingHttpPublicKeyPinPolicy",
    "platformSettingHttpPublicKeyPinPolicyReportOnlyEnabled",
    "platformSettingHttpStrictTransportEnabled",
    "platformSettingLoadBalancerHeartbeatAddress",
    "platformSettingLoadBalancerHeartbeatPort",
    "platformSettingLoadBalancerManagerAddress",
    "platformSettingLoadBalancerManagerPort",
    "platformSettingLoadBalancerRelayAddress",
    "platformSettingLoadBalancerRelayPort",
    "platformSettingLogoBinaryImageImg",
    "platformSettingNewTenantDownloadSecurityUpdateEnabled",
    "platformSettingPrimaryTenantAllowTenantAddVmwareVcenterEnabled",
    "platformSettingPrimaryTenantAllowTenantConfigureForgotPasswordEnabled",
    "platformSettingPrimaryTenantAllowTenantConfigureRememberMeOptionEnabled",
    "platformSettingPrimaryTenantAllowTenantConfigureSiemEnabled",
    "platformSettingPrimaryTenantAllowTenantConfigureSnmpEnabled",
    "platformSettingPrimaryTenantAllowTenantConfigureSnsEnabled",
    "platformSettingPrimaryTenantAllowTenantControlImpersonationEnabled",
    "platformSettingPrimaryTenantAllowTenantDatabaseState",
    "platformSettingPrimaryTenantAllowTenantRunComputerDiscoveryEnabled",
    "platformSettingPrimaryTenantAllowTenantRunPortScanEnabled",
    "platformSettingPrimaryTenantAllowTenantSyncWithCloudAccountEnabled",
    "platformSettingPrimaryTenantAllowTenantSynchronizeLdapDirectoriesEnabled",
    "platformSettingPrimaryTenantAllowTenantUseDefaultRelayGroupEnabled",
    "platformSettingPrimaryTenantAllowTenantUseScheduledRunScriptTaskEnabled",
    "platformSettingPrimaryTenantLockAndHideTenantDataPrivacyOptionEnabled",
    "platformSettingPrimaryTenantLockAndHideTenantSmtpTabEnabled",
    "platformSettingPrimaryTenantLockAndHideTenantStorageTabEnabled",
    "platformSettingPrimaryTenantShareConnectedThreatDefensesEnabled",
    "platformSettingPrimaryTenantShareManagedDetectResponsesEnabled",
    "platformSettingProductUsageDataCollectionEnabled",
    "platformSettingProxyManagerCloudProxyId",
    "platformSettingProxyManagerUpdateProxyId",
    "platformSettingRecommendationCpuUsageLevel",
    "platformSettingRetainServerLogDuration",
    "platformSettingSamlServiceProviderCertificate",
    "platformSettingSamlServiceProviderCertificateExpiryWarningDays",
    "platformSettingSamlServiceProviderEntityId",
    "platformSettingSamlServiceProviderName",
    "platformSettingSamlServiceProviderPrivateKey",
    "platformSettingSignInPageMessage",
    "platformSettingTenantProtectionUsageMonitoringComputerId1",
    "platformSettingTenantProtectionUsageMonitoringComputerId2",
    "platformSettingTenantProtectionUsageMonitoringComputerId3",
    "platformSettingUpdateAgentSoftwareUseDownloadCenterOnMissingDeepSecurityManagerEnabled",
    "platformSettingUpdateImportedSoftwareAutoDownloadEnabled",
    "platformSettingUserEnforceTermsAndConditionsEnabled",
    "platformSettingUserEnforceTermsAndConditionsMessage",
    "platformSettingUserEnforceTermsAndConditionsTitle",
    "platformSettingVmwareNsxManagerNode",
    "platformSettingDsmAsXbcAgentFeatureEnabled",
    "platformSettingUpgradeOnActivationEnabled",
    "firewallSettingGlobalStatefulConfigId",
]


def TransferSettings(settings_dict, no_fly_list, stateful_dict, tenant2key):
    print("Transferring system settings...", flush=True)
    sanitized_settings_dict = {}
    for setting, value_dict in settings_dict.items():
        if setting not in no_fly_list:
            sanitized_settings_dict[setting] = value_dict
    try:
        SystemSettingsApiInstance(tenant2key).modify(sanitized_settings_dict)
    except ApiException as e:
        print(e.body)
        pass


def settings_transfer(
    OLD_HOST, OLD_API_KEY, NEW_HOST, NEW_API_KEY, stateful_dict, no_fly_list=no_fly_list
):
    settings_dict = ListSettings(OLD_HOST, OLD_API_KEY)
    TransferSettings(settings_dict, no_fly_list, stateful_dict, NEW_API_KEY)



import os
import sys
import datetime
import deepsecurity
import time
import requests
import urllib3
import traceback
import logging
import click
from datetime import datetime
import dsmigrator.api_config
from dsmigrator.policies import (
    ListAllPolicy,
    GetPolicy,
    AddPolicy,
    delete_cloud_one_policies,
)
from dsmigrator.proxy import proxy_edit
from dsmigrator.ips import ips_rules_transform
from dsmigrator.antimalware import am_config_transform, am_validate_create
from dsmigrator.integrity import im_config_transform
from dsmigrator.loginspection import li_config_transform
from dsmigrator.firewall import firewall_config_transform
from dsmigrator.lists import (
    directory_listmaker,
    port_listmaker,
    mac_listmaker,
    ip_listmaker,
    stateful_listmaker,
    context_listmaker,
    schedule_listmaker,
)
from dsmigrator.tasks import ebt_listmaker, st_listmaker
from dsmigrator.computer_groups import computer_group_listmaker
from dsmigrator.system_settings import settings_transfer
import yaml


class Logger(object):
    def flush(self):
        pass

    def __init__(self):
        self.terminal = sys.stdout
        filename = datetime.now().strftime("migrator_%H_%M_%d_%m_%Y.log")
        self.log = open(f"./{filename}", "a")

    def write(self, message):
        self.terminal.write(message)
        self.log.write(message)


# override the click invoke method
def CommandWithConfigFile(config_file_param_name):
    class CustomCommandClass(click.Command):
        def invoke(self, ctx):
            config_file = ctx.params[config_file_param_name]
            if config_file is not None:
                with open(config_file) as f:
                    config_data = yaml.safe_load(f)
                    for param, value in ctx.params.items():
                        if value is None and param in config_data:
                            ctx.params[param] = config_data[param]
                            ctx.params[param].prompt = None

            return super(CustomCommandClass, self).invoke(ctx)

    return CustomCommandClass


@click.command(cls=CommandWithConfigFile("config_file"))
@click.option("--config-file", type=click.Path(), is_eager=True)
@click.option(
    "-ou",
    "--original-url",
    prompt="Old DSM url",
    help="A resolvable FQDN for the old DSM, with port number (e.g. https://192.168.1.1:4119)",
    envvar="ORIGINAL_URL",
)
@click.option(
    "-oa",
    "--original-api-key",
    prompt="Old DSM API key",
    hide_input=True,
    help="API key for the old DSM with Full Access permissions",
    envvar="ORIGINAL_API_KEY",
)
@click.option(
    "-nu",
    "--new-url",
    default="https://cloudone.trendmicro.com/",
    show_default=True,
    help="Destination url",
)
@click.option(
    "-coa",
    "--cloud-one-api-key",
    prompt="New Cloud One API key",
    hide_input=True,
    help="API key for Cloud One Workload Security with Full Access permissions",
    envvar="CLOUD_ONE_API_KEY",
)
@click.option(
    "-d",
    "--delete-policies/--keep-policies",
    is_flag=True,
    default=False,
    prompt="Do you want to wipe existing policies in Cloud One? (not required, but will give best results)",
    help="Wipes existing policies in Cloud One (not required, but will give best results)",
)
@click.option(
    "-t",
    "--tasks",
    is_flag=True,
    help="(BETA) Enable the task migrator (may be buggy)",
)
@click.option(
    "-k",
    "--insecure",
    is_flag=True,
    help="Suppress the InsecureRequestWarning for self-signed certificates",
)
@click.option(
    "-c",
    "--cert",
    default=False,
    show_default=True,
    help="(Optional) Allows the use of a cert file",
)
def main(
    config_file,
    original_url,
    original_api_key,
    new_url,
    cloud_one_api_key,
    cert,
    insecure,
    tasks,
    delete_policies,
):
    """Moves your on-prem DS deployment to the cloud!"""
    sys.stdout = Logger()
    sys.stderr = sys.stdout

    OLD_API_KEY = original_api_key
    OLD_HOST = original_url
    NEW_API_KEY = cloud_one_api_key
    NEW_HOST = new_url

    if insecure:
        urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

    if delete_policies:
        delete_cloud_one_policies(NEW_API_KEY)

    old_policy_name_enum, old_policy_id_list = ListAllPolicy(OLD_HOST, OLD_API_KEY)

    antimalwareconfig, allofpolicy = GetPolicy(
        old_policy_id_list, OLD_HOST, OLD_API_KEY
    )

    amdirectorylist, amfileextensionlist, amfilelist, allamconfig = am_config_transform(
        antimalwareconfig, OLD_HOST, OLD_API_KEY
    )

    amalldirectorynew, amallfileextentionnew, amallfilelistnew = directory_listmaker(
        amdirectorylist,
        amfileextensionlist,
        amfilelist,
        OLD_HOST,
        OLD_API_KEY,
        NEW_API_KEY,
    )

    t1portlistall, t1portlistname, t1portlistid, t2portlistid = port_listmaker(
        OLD_HOST, OLD_API_KEY, NEW_HOST, NEW_API_KEY
    )
    t1maclistall, t1maclistname, t1maclistid, t2maclistid = mac_listmaker(
        OLD_HOST, OLD_API_KEY, NEW_HOST, NEW_API_KEY
    )
    t1iplistall, t1iplistname, t1iplistid, t2iplistid = ip_listmaker(
        OLD_HOST, OLD_API_KEY, NEW_HOST, NEW_API_KEY
    )
    (
        t1statefulall,
        t1statefulname,
        t1statefulid,
        t2statefulid,
        stateful_dict,
    ) = stateful_listmaker(OLD_HOST, OLD_API_KEY, NEW_HOST, NEW_API_KEY)
    t1contextid, t2contextid = context_listmaker(
        OLD_HOST, OLD_API_KEY, NEW_HOST, NEW_API_KEY
    )
    t1scheduleid, t2scheduleid = schedule_listmaker(
        OLD_HOST, OLD_API_KEY, NEW_HOST, NEW_API_KEY
    )

    print(stateful_dict)
    try:
        settings_transfer(OLD_HOST, OLD_API_KEY, NEW_HOST, NEW_API_KEY, stateful_dict)
    except Exception:
        pass

    proxy_edit(allofpolicy, t1iplistid, t2iplistid, t1portlistid, t2portlistid)

    # TRANSFORM
    allofpolicy = ips_rules_transform(
        allofpolicy,
        t1portlistid,
        t2portlistid,
        t1scheduleid,
        t2scheduleid,
        t1contextid,
        t2contextid,
        OLD_HOST,
        OLD_API_KEY,
        NEW_HOST,
        NEW_API_KEY,
    )
    allofpolicy = am_validate_create(
        allofpolicy,
        antimalwareconfig,
        allamconfig,
        amdirectorylist,
        amalldirectorynew,
        amfileextensionlist,
        amallfileextentionnew,
        amfilelist,
        amallfilelistnew,
        NEW_HOST,
        NEW_API_KEY,
    )
    allofpolicy = im_config_transform(
        allofpolicy, OLD_HOST, OLD_API_KEY, NEW_HOST, NEW_API_KEY
    )
    allofpolicy = li_config_transform(
        allofpolicy, OLD_HOST, OLD_API_KEY, NEW_HOST, NEW_API_KEY
    )
    allofpolicy = firewall_config_transform(
        allofpolicy,
        t1iplistid,
        t2iplistid,
        t1maclistid,
        t2maclistid,
        t1portlistid,
        t2portlistid,
        t1statefulid,
        t2statefulid,
        t1scheduleid,
        t2scheduleid,
        t1contextid,
        t2contextid,
        OLD_HOST,
        OLD_API_KEY,
        NEW_HOST,
        NEW_API_KEY,
    )
    policy_dict = AddPolicy(allofpolicy, NEW_API_KEY)
    if tasks:
        computer_group_dict = computer_group_listmaker(
            OLD_HOST, OLD_API_KEY, NEW_HOST, NEW_API_KEY
        )
        ebt_listmaker(
            policy_dict,
            computer_group_dict,
            OLD_HOST,
            OLD_API_KEY,
            NEW_HOST,
            NEW_API_KEY,
        )
        st_listmaker(
            policy_dict,
            computer_group_dict,
            OLD_HOST,
            OLD_API_KEY,
            NEW_HOST,
            NEW_API_KEY,
        )


if __name__ == "__main__":
    main()  # pylint: disable=no-value-for-parameter



import json
import urllib3
import requests

cert = False


def ips_rules_transform(
    allofpolicy,
    t1portlistid,
    t2portlistid,
    t1scheduleid,
    t2scheduleid,
    t1contextid,
    t2contextid,
    OLD_HOST,
    OLD_API_KEY,
    NEW_HOST,
    NEW_API_KEY,
):
    og_ipsruleid = IPSGet(allofpolicy)
    og_ipsappid = IPSappGet(allofpolicy)

    allipsapp, allipsappidnew1, allipsappidold, allipscustomapp = IPSappDescribe(
        og_ipsappid,
        t1portlistid,
        t2portlistid,
        OLD_HOST,
        OLD_API_KEY,
        NEW_HOST,
        NEW_API_KEY,
    )

    allipsappidnew2 = IPSappCustom(allipsapp, allipscustomapp, NEW_HOST, NEW_API_KEY)

    allipsrule, allipsruleidnew1, allipsruleidold, allipscustomrule = IPSDescribe(
        og_ipsruleid,
        og_ipsappid,
        allipsappidnew1,
        allipsappidnew2,
        allipsappidold,
        allipscustomapp,
        t1scheduleid,
        t2scheduleid,
        t1contextid,
        t2contextid,
        OLD_HOST,
        OLD_API_KEY,
        NEW_HOST,
        NEW_API_KEY,
    )

    allipsruleidnew2 = IPSCustom(allipsrule, allipscustomrule, NEW_HOST, NEW_API_KEY)

    aop_replace_ips_rules = IPSReplace(
        allofpolicy,
        allipsruleidnew1,
        allipsruleidnew2,
        og_ipsruleid,
        allipsruleidold,
        allipscustomrule,
    )
    aop_replace_ips_apps = IPSappReplace(
        aop_replace_ips_rules,
        allipsappidnew1,
        allipsappidnew2,
        og_ipsappid,
        allipsappidold,
        allipscustomapp,
    )
    final = aop_replace_ips_apps
    return final


def IPSappGet(allofpolicy):
    ipsappid = []
    print("IPS application types in Tenant 1", flush=True)
    for describe in allofpolicy:
        namejson = json.loads(describe)
        if "applicationTypeIDs" in namejson["intrusionPrevention"]:
            for count, here2 in enumerate(
                namejson["intrusionPrevention"]["applicationTypeIDs"]
            ):
                ipsappid.append(str(here2))
    ipsappid = list(dict.fromkeys(ipsappid))
    print(ipsappid, flush=True)
    return ipsappid


def IPSappDescribe(
    ipsappid,
    t1portlistid,
    t2portlistid,
    url_link_final,
    tenant1key,
    url_link_final_2,
    tenant2key,
):
    allipsapp = []
    allipsappname = []
    allipsappidnew1 = []
    allipsappidold = []
    allipscustomapp = []
    print("Searching IPS application types in Tenant 1...", flush=True)
    if ipsappid:
        for count, dirlist in enumerate(ipsappid):
            payload = {}
            url = url_link_final + "api/applicationtypes/" + str(dirlist)
            headers = {
                "api-secret-key": tenant1key,
                "api-version": "v1",
                "Content-Type": "application/json",
            }
            response = requests.request(
                "GET", url, headers=headers, data=payload, verify=cert
            )
            describe = str(response.text)
            allipsapp.append(describe)
            ipsappjson = json.loads(describe)
            allipsappname.append(str(ipsappjson["name"]))
            print(
                "#"
                + str(count)
                + " IPS Application Type name: "
                + str(ipsappjson["name"]),
                flush=True,
            )
            index3 = describe.find("portListID")
            if index3 != -1:
                indexpart = describe[index3 + 10 :]
                startIndex = indexpart.find(":")
                if startIndex != -1:  # i.e. if the first quote was found
                    endIndex3 = indexpart.find(",", startIndex + 1)
                    if (
                        startIndex != -1 and endIndex3 != -1
                    ):  # i.e. both quotes were found
                        indexid1 = indexpart[startIndex + 1 : endIndex3]
                        indexid5 = describe[index3 : index3 + 10 + endIndex3]
                        indexnum = t1portlistid.index(indexid1)
                        listpart = indexid5.replace(indexid1, t2portlistid[indexnum])
                        allipsapp[count] = describe.replace(indexid5, listpart)
            print("#" + str(count) + " IPS Application Type ID: " + dirlist, flush=True)
    print("Done!", flush=True)
    print("Searching and Modifying IPS application types in Tenant 2...", flush=True)
    for count, dirlist in enumerate(allipsappname):
        payload = (
            '{"searchCriteria": [{"fieldName": "name","stringValue": "'
            + dirlist
            + '"}]}'
        )
        url = url_link_final_2 + "api/applicationtypes/search"
        headers = {
            "api-secret-key": tenant2key,
            "api-version": "v1",
            "Content-Type": "application/json",
        }
        response = requests.request(
            "POST", url, headers=headers, data=payload, verify=cert
        )
        describe = str(response.text)
        taskjson = json.loads(describe)
        if not "message" in taskjson:
            index = describe.find(dirlist)
            if index != -1:
                index = describe.find('"ID"')
                if index != -1:
                    indexpart = describe[index + 4 :]
                    startIndex = indexpart.find(":")
                    if startIndex != -1:  # i.e. if the first quote was found
                        endIndex = indexpart.find("}", startIndex + 1)
                        if (
                            startIndex != -1 and endIndex != -1
                        ):  # i.e. both quotes were found
                            indexid = indexpart[startIndex + 1 : endIndex]
                            allipsappidnew1.append(str(indexid))
                            allipsappidold.append(count)
                            payload = allipsapp[count]
                            url = (
                                url_link_final_2
                                + "api/applicationtypes/"
                                + str(indexid)
                            )
                            headers = {
                                "api-secret-key": tenant2key,
                                "api-version": "v1",
                                "Content-Type": "application/json",
                            }
                            response = requests.request(
                                "POST", url, headers=headers, data=payload, verify=cert
                            )
                            print(
                                "#"
                                + str(count)
                                + " IPS Application Type ID: "
                                + indexid,
                                flush=True,
                            )
                else:
                    print(describe, flush=True)
                    print(payload, flush=True)
            else:
                allipscustomapp.append(count)
        else:
            print(describe, flush=True)
            print(payload, flush=True)
    print("Done!", flush=True)
    return allipsapp, allipsappidnew1, allipsappidold, allipscustomapp


def IPSappCustom(allipsapp, allipscustomapp, url_link_final_2, tenant2key):
    allipsappidnew2 = []
    if allipscustomapp:
        print("Creating IPS application Type Custom Rule...", flush=True)
        for count, indexnum in enumerate(allipscustomapp):
            payload = allipsapp[indexnum]
            url = url_link_final_2 + "api/applicationtypes"
            headers = {
                "api-secret-key": tenant2key,
                "api-version": "v1",
                "Content-Type": "application/json",
            }
            response = requests.request(
                "POST", url, headers=headers, data=payload, verify=cert
            )
            describe = str(response.text)
            index = describe.find('"ID"')
            if index != -1:
                indexpart = describe[index + 4 :]
                startIndex = indexpart.find(":")
                if startIndex != -1:  # i.e. if the first quote was found
                    endIndex = indexpart.find("}", startIndex + 1)
                    if (
                        startIndex != -1 and endIndex != -1
                    ):  # i.e. both quotes were found
                        indexid = indexpart[startIndex + 1 : endIndex]
                        allipsappidnew2.append(str(indexid))
                        print(
                            "#"
                            + str(count)
                            + " IPS Application Type ID: "
                            + str(indexid),
                            flush=True,
                        )
            else:
                print(describe, flush=True)
                print(payload, flush=True)
        print("Done!", flush=True)
    return allipsappidnew2


def IPSappReplace(
    allofpolicy,
    allipsappidnew1,
    allipsappidnew2,
    ipsappid,
    allipsappidold,
    allipscustomapp,
):
    for count, describe in enumerate(allofpolicy):
        taskjson = json.loads(describe)
        if "applicationTypeIDs" in taskjson["intrusionPrevention"]:
            if allipsappidnew1 or allipsappidnew2:
                for count1, this in enumerate(
                    taskjson["intrusionPrevention"]["applicationTypeIDs"]
                ):
                    checkindex = ipsappid.index(str(this))
                    if checkindex in allipsappidold:
                        checkindex1 = allipsappidold.index(checkindex)
                        taskjson["intrusionPrevention"]["applicationTypeIDs"][
                            count1
                        ] = allipsappidnew1[checkindex1]
                    elif checkindex in allipscustomapp:
                        checkindex1 = allipscustomapp.index(checkindex)
                        taskjson["intrusionPrevention"]["applicationTypeIDs"][
                            count1
                        ] = allipsappidnew2[checkindex1]
        allofpolicy[count] = json.dumps(taskjson)
    return allofpolicy


def IPSGet(allofpolicy):
    ipsruleid = []
    print("IPS rules in Tenant 1", flush=True)
    for describe in allofpolicy:
        namejson = json.loads(describe)
        if "ruleIDs" in namejson["intrusionPrevention"]:
            for count, here2 in enumerate(namejson["intrusionPrevention"]["ruleIDs"]):
                ipsruleid.append(str(here2))
    ipsruleid = list(dict.fromkeys(ipsruleid))
    print(ipsruleid, flush=True)
    return ipsruleid


def IPSDescribe(
    ipsruleid,
    ipsappid,
    allipsappidnew1,
    allipsappidnew2,
    allipsappidold,
    allipscustomapp,
    t1scheduleid, t2scheduleid, t1contextid, t2contextid,
    url_link_final,
    tenant1key,
    url_link_final_2,
    tenant2key,
):
    allipsrule = []
    allipsrulename = []
    allipsruleidnew1 = []
    allipsruleidold = []
    allipscustomrule = []
    print("Searching IPS rules in Tenant 1...", flush=True)
    if ipsruleid:
        for count, dirlist in enumerate(ipsruleid):
            payload = {}
            url = url_link_final + "api/intrusionpreventionrules/" + str(dirlist)
            headers = {
                "api-secret-key": tenant1key,
                "api-version": "v1",
                "Content-Type": "application/json",
            }
            response = requests.request(
                "GET", url, headers=headers, data=payload, verify=cert
            )
            describe = str(response.text)
            allipsrule.append(describe)
            ipsjson = json.loads(describe)
            allipsrulename.append(str(ipsjson["name"]))
            print(
                "#" + str(count) + " IPS rule name: " + str(ipsjson["name"]), flush=True
            )
            index3 = describe.find("applicationTypeID")
            if index3 != -1:
                indexpart = describe[index3 + 17 :]
                startIndex = indexpart.find(":")
                if startIndex != -1:  # i.e. if the first quote was found
                    endIndex3 = indexpart.find(",", startIndex + 1)
                    if (
                        startIndex != -1 and endIndex3 != -1
                    ):  # i.e. both quotes were found
                        indexid1 = indexpart[startIndex + 1 : endIndex3]
                        checkindex = ipsappid.index(indexid1)
                        if checkindex in allipsappidold:
                            checkindex1 = allipsappidold.index(checkindex)
                            replaceid = allipsappidnew1[checkindex1]
                        elif checkindex in allipscustomapp:
                            checkindex1 = allipscustomapp.index(checkindex)
                            replaceid = allipsappidnew2[checkindex1]
                        indexid5 = describe[index3 : index3 + 17 + endIndex3]
                        listpart = indexid5.replace(indexid1, replaceid)
                        allipsrule[count] = describe.replace(indexid5, listpart)
            ipsjson = json.loads(allipsrule[count])
            if 'scheduleID' in ipsjson:
                indexnum = t1scheduleid.index(str(ipsjson['scheduleID']))
                ipsjson['scheduleID'] = t2scheduleid[indexnum]
            if 'contextID' in ipsjson:
                indexnum = t1contextid.index(str(ipsjson['contextID']))
                ipsjson['contextID'] = t2contextid[indexnum]
            allipsrule[count] = json.dumps(ipsjson)
            print("#" + str(count) + " IPS rule ID: " + dirlist, flush=True)
    print("Done!", flush=True)
    print("Searching and Modifying IPS rule in Tenant 2...", flush=True)
    for count, dirlist in enumerate(allipsrulename):
        payload = (
            '{"searchCriteria": [{"fieldName": "name","stringValue": "'
            + dirlist
            + '"}]}'
        )
        url = url_link_final_2 + "api/intrusionpreventionrules/search"
        headers = {
            "api-secret-key": tenant2key,
            "api-version": "v1",
            "Content-Type": "application/json",
        }
        response = requests.request(
            "POST", url, headers=headers, data=payload, verify=cert
        )
        describe = str(response.text)
        taskjson = json.loads(describe)
        if not "message" in taskjson:
            index = describe.find(dirlist)
            if index != -1:
                index = describe.find('"ID"')
                if index != -1:
                    indexpart = describe[index + 4 :]
                    startIndex = indexpart.find(":")
                    if startIndex != -1:  # i.e. if the first quote was found
                        endIndex = indexpart.find(",", startIndex + 1)
                        if (
                            startIndex != -1 and endIndex != -1
                        ):  # i.e. both quotes were found
                            indexid = indexpart[startIndex + 1 : endIndex]
                            allipsruleidnew1.append(str(indexid))
                            allipsruleidold.append(count)
                            print(
                                "#" + str(count) + " IPS rule ID: " + str(indexid),
                                flush=True,
                            )
                        else:
                            endIndex = indexpart.find("}", startIndex + 1)
                            if (
                                startIndex != -1 and endIndex != -1
                            ):  # i.e. both quotes were found
                                indexid = indexpart[startIndex + 1 : endIndex]
                                allipsruleidnew1.append(str(indexid))
                                allipsruleidold.append(count)
                                print(
                                    "#" + str(count) + " IPS rule ID: " + str(indexid),
                                    flush=True,
                                )
                else:
                    print(describe, flush=True)
                    print(payload, flush=True)
            else:
                allipscustomrule.append(count)
        else:
            print(describe, flush=True)
            print(payload, flush=True)
    print("Done!", flush=True)
    # print("Tenant 2 default IPS rules", flush=True)
    # print(allipsruleidnew1, flush=True)
    return allipsrule, allipsruleidnew1, allipsruleidold, allipscustomrule


def IPSCustom(allipsrule, allipscustomrule, url_link_final_2, tenant2key):
    allipsruleidnew2 = []
    if allipscustomrule:
        print("Creating new custom IPS rule in Tenant 2...", flush=True)
        for count, indexnum in enumerate(allipscustomrule):
            payload = allipsrule[indexnum]
            url = url_link_final_2 + "api/intrusionpreventionrules"
            headers = {
                "api-secret-key": tenant2key,
                "api-version": "v1",
                "Content-Type": "application/json",
            }
            response = requests.request(
                "POST", url, headers=headers, data=payload, verify=cert
            )
            describe = str(response.text)
            index = describe.find('"ID"')
            if index != -1:
                indexpart = describe[index + 4 :]
                startIndex = indexpart.find(":")
                if startIndex != -1:  # i.e. if the first quote was found
                    endIndex = indexpart.find(",", startIndex + 1)
                    if (
                        startIndex != -1 and endIndex != -1
                    ):  # i.e. both quotes were found
                        indexid = indexpart[startIndex + 1 : endIndex]
                        allipsruleidnew2.append(str(indexid))
                        print(
                            "#" + str(count) + " IPS rule ID: " + str(indexid),
                            flush=True,
                        )
                    else:
                        endIndex = indexpart.find("}", startIndex + 1)
                        if (
                            startIndex != -1 and endIndex != -1
                        ):  # i.e. both quotes were found
                            indexid = indexpart[startIndex + 1 : endIndex]
                            allipsruleidnew2.append(str(indexid))
                            print(
                                "#" + str(count) + " IPS rule ID: " + str(indexid),
                                flush=True,
                            )
            else:
                print(describe, flush=True)
                print(payload, flush=True)
        print("Done!", flush=True)
    # print("all new IPS rule custom rule", flush=True)
    # print(allipsruleidnew2, flush=True)
    return allipsruleidnew2


def IPSReplace(
    allofpolicy,
    allipsruleidnew1,
    allipsruleidnew2,
    ipsruleid,
    allipsruleidold,
    allipscustomrule,
):
    for count, describe in enumerate(allofpolicy):
        index = describe.find('"intrusionPrevention"')
        if index != -1:
            indexpart = describe[index + 20 :]
            startIndex = indexpart.find("}")
            if startIndex != -1:  # i.e. if the first quote was found
                endIndex = indexpart.find("}", startIndex + 1)
                if startIndex != -1 and endIndex != -1:  # i.e. both quotes were found
                    indexid = indexpart[startIndex + 1 : endIndex]
                    index2 = indexid.find("ruleIDs")
                    if index2 != -1:
                        indexpart2 = indexid[index2 + 9 :]
                        startIndex2 = indexpart2.find("[")
                        if startIndex2 != -1:  # i.e. if the first quote was found
                            endIndex2 = indexpart2.find("]", startIndex2 + 1)
                            if (
                                startIndex2 != -1 and endIndex2 != -1
                            ):  # i.e. both quotes were found
                                indexid2 = indexpart2[startIndex2 + 1 : endIndex2]
                                indexid3 = indexpart2[startIndex2 + 1 : endIndex2]
                                indexid2.replace(", ", ",")
                                indexid4 = indexid2.split(",")
                                if allipsruleidnew1 or allipsruleidnew2:
                                    for count1, this in enumerate(indexid4):
                                        checkindex = ipsruleid.index(this)
                                        if checkindex in allipsruleidold:
                                            checkindex1 = allipsruleidold.index(
                                                checkindex
                                            )
                                            indexid4[count1] = allipsruleidnew1[
                                                checkindex1
                                            ]
                                        elif checkindex in allipscustomrule:
                                            checkindex1 = allipscustomrule.index(
                                                checkindex
                                            )
                                            indexid4[count1] = allipsruleidnew2[
                                                checkindex1
                                            ]
                                    indexid2 = ",".join(indexid4)
                                modulepart = describe[index : index + 20 + endIndex]
                                modulepart2 = modulepart.replace(indexid3, indexid2)
                                allofpolicy[count] = describe.replace(
                                    modulepart, modulepart2
                                )
    return allofpolicy
